<?php
namespace Bitfoundry\Etnpay\Controller\Scan;

class Success extends \Bitfoundry\Etnpay\Controller\AbstractEtnpay
{
    public function execute()
    {
        try
        {
            $order = $this->_checkoutSession->getLastRealOrder();
            $orderId = $order->getId();
            $paymentId = $order->getRealOrderId();
            $helper = $this->helper;
            $etnpay = $this->_paymentMethod;

            // check ETN payment status once again for good measure
            $etnStatus = $etnpay->getEtnStatus($paymentId);
            if ($etnStatus['status'] == 1) {
                if ($order->getBaseTotalDue() != 0) { // prevent double invoicing because of the payment webhook
                    $invoice = $order->prepareInvoice();
                    $invoice->setEtnAmount($etnStatus['amount']);
                    // $invoice->setEtnId($etnStatus['ref']); // not implemented yet on electroneum side
                    $invoice->register()->capture();

                    $transaction = $this->_transactionFactory->create();
                    $transaction->addObject($invoice)
                        ->addObject($invoice->getOrder())
                        ->save();

                    $status = \Magento\Sales\Model\Order::STATE_PROCESSING;

                    $order->setStatus( $status ); //configure the status
                    $order->setState( $status )->save(); //try and configure the status
                    $order->addStatusHistoryComment(__( 'Invoice created #%1.', $invoice->getIncrementId()));
                    $order->setIsCustomerNotified(true);
                    $order->save();

                    $this->messageManager->addSuccessMessage('Electronuem Payment for ' .$etnStatus['amount']. ' Successful.');
                }

                $this->_redirect('checkout/onepage/success', array('_secure'=> true));
            } // end if
            else {
                $this->_redirect('checkout/onepage/failure', array('_secure'=> true));
                $this->_logger->error("************************************************************************************");
                $this->_logger->error("*    Payment Tampering");
                $this->_logger->error("*    Looks like we have someone trying to redirect straight to the success page on order: " . $order->getIncrementId() . ", Electroneum Instant Payments.");
                $this->_logger->error("************************************************************************************");
            }
        } catch ( \Magento\Framework\Exception\LocalizedException $e ) {
            $this->_logger->error($e->getMessage());
            $this->messageManager->addExceptionMessage($e, $e->getMessage());
            $this->_redirect('checkout/cart', array('_secure'=> true));
        } catch (\Exception $e) {
            $this->_logger->error($e->getMessage());
            $this->messageManager->addExceptionMessage($e, __('We can\'t start Electroneum Checkout.'));
            $this->_redirect('checkout/cart', array('_secure'=> true));
        }

        return '';
    }
}
