<?php
namespace Bitfoundry\Etnpay\Helper;

class Data extends \Magento\Framework\App\Helper\AbstractHelper
{
    protected static $_shouldAskToCreateBillingAgreement = false;
    protected $_paymentData;
    private $methodCodes;
    private $configFactory;
    protected $_logger;
    protected $mobileDetect;

    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        \Magento\Payment\Helper\Data $paymentData,
        \Magento\Framework\App\Config\BaseFactory $configFactory,
        \Bitfoundry\Etnpay\Helper\MobileDetect $mobileDetect,
        array $methodCodes
    ) {
        $this->_logger = $context->getLogger();
        $this->_paymentData  = $paymentData;
        $this->methodCodes   = $methodCodes;
        $this->configFactory = $configFactory;
        $this->mobileDetect  = $mobileDetect;

        parent::__construct( $context );
    }

    public function shouldAskToCreateBillingAgreement()
    {
        return self::$_shouldAskToCreateBillingAgreement;
    }

    public function getBillingAgreementMethods( $store = null, $quote = null )
    {
        $result = [];
        foreach ( $this->_paymentData->getStoreMethods( $store, $quote ) as $method ) {
            if ( $method instanceof MethodInterface ) {
                $result[] = $method;
            }
        }

        return $result;
    }

    public function isMobile()
    {
        $mobileDetect = new $this->mobileDetect;
        if ($mobileDetect->isMobile()) {
            return true;
        }

        return false;
    }

    public function createSecuritySignature($payloadArr, $apiSecret)
    {
        $hash = hash_hmac('sha256', json_encode($payloadArr), $apiSecret);

        return $hash;
    }


    public function getIp()
    {
        //Just get the headers if we can or else use the SERVER global
        if ( function_exists( 'apache_request_headers' ) ) {
            $headers = apache_request_headers();
        } else {
            $headers = $_SERVER;
        }
        //Get the forwarded IP if it exists
        if ( array_key_exists( 'X-Forwarded-For', $headers ) && filter_var( $headers['X-Forwarded-For'], FILTER_VALIDATE_IP, FILTER_FLAG_IPV4 ) ) {
            $the_ip = $headers['X-Forwarded-For'];
        } elseif ( array_key_exists( 'HTTP_X_FORWARDED_FOR', $headers ) && filter_var( $headers['HTTP_X_FORWARDED_FOR'], FILTER_VALIDATE_IP, FILTER_FLAG_IPV4 )
        ) {
            $the_ip = $headers['HTTP_X_FORWARDED_FOR'];
        } else {
            $the_ip = filter_var( $_SERVER['REMOTE_ADDR'], FILTER_VALIDATE_IP, FILTER_FLAG_IPV4 );
        }

        return $the_ip;
    }

    // get the value of ETN required for payment
    public function getETNAmount($currencyCode, $amount) {
        // Get the JSON conversion data.
        if (!$json = file_get_contents('https://supply.electroneum.com/app-value-v2.json')) {
            die("Could not load currency exchange JSON");
        }

        // Check the JSON is valid.
        $arr = json_decode($json, true);
        if (json_last_error() !== JSON_ERROR_NONE) {
            die("Could not load valid currency exchange JSON");
        }

        // Get the conversion rate.
        if (!$rate = $arr['price_' . strtolower($currencyCode)]) {
            die("Currency exchange rate not found");
        }

        // Check the rate is valid or more than zero.
        if (floatval($rate) <= 0) {
            die("Currency conversion rate not valid");
        }

        // Calculate the amount of ETN.
        $etn = number_format(floatval($amount) / $rate, 2, '.', '');

        return $etn;
    }

    // get the value of ETN required for payment
    public function getEtnValue() {
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $storeManager = $objectManager->get('\Magento\Store\Model\StoreManagerInterface');
        $store = $storeManager->getStore();
        $currencyCode  = $store->getCurrentCurrencyCode();

        // Get the JSON conversion data.
        if (!$json = file_get_contents('https://supply.electroneum.com/app-value-v2.json')) {
            die("Could not load currency exchange JSON");
        }

        // Check the JSON is valid.
        $arr = json_decode($json, true);
        if (json_last_error() !== JSON_ERROR_NONE) {
            die("Could not load valid currency exchange JSON");
        }

        // Get the conversion rate.
        if (!$rate = $arr['price_' . strtolower($currencyCode)]) {
            die("Currency exchange rate not found");
        }

        // Check the rate is valid or more than zero.
        if (floatval($rate) <= 0) {
            die("Currency conversion rate not valid");
        }

        // Calculate the amount of ETN.
        $etnValue = number_format(floatval(1.0) / $rate, 2, '.', '');

        return $etnValue;
    }

    public function getEtnAmountDetails() {
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $storeManager = $objectManager->get('\Magento\Store\Model\StoreManagerInterface');
        $cart = $objectManager->get('\Magento\Checkout\Model\Cart');

        $store = $storeManager->getStore();
        $currencyCode  = $store->getCurrentCurrencyCode();
        $shippingTotal = $cart->getQuote()->getShippingAddress()->getShippingAmount();
        $this->_logger->debug('shippingTotal: ' . $shippingTotal);
        $grandTotal = $cart->getQuote()->getGrandTotal();
        //$combinedTotal = $grandTotal + $shippingTotal;

        $details = 'Order Total: (approx) ' . $this->getETNAmount($currencyCode, $grandTotal) . 'ETN';

        return $details;
    }
}
