<?php
namespace Bitfoundry\Etnpay\Model;

use Magento\Checkout\Model\ConfigProviderInterface;
use Magento\Customer\Helper\Session\CurrentCustomer;
use Magento\Framework\Locale\ResolverInterface;
use Magento\Payment\Helper\Data as PaymentHelper;
use Bitfoundry\Etnpay\Helper\Data as EtnpayHelper;

class EtnpayConfigProvider implements ConfigProviderInterface
{
    //const CODE = 'etnpay';
    /**
     * @var ResolverInterface
     */
    protected $localeResolver;

    /**
     * @var Config
     */
    protected $config;

    /**
     * @var \Magento\Customer\Helper\Session\CurrentCustomer
     */
    protected $currentCustomer;

    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $_logger;

    /**
     * @var EtnpayHelper
     */
    protected $etnpayHelper;

    /**
     * @var string[]
     */
     protected $methodCodes = [
         Config::METHOD_CODE,
     ];

    /**
     * @var \Magento\Payment\Model\Method\AbstractMethod[]
     */
    protected $methods = [];

    /**
     * @var PaymentHelper
     */
    protected $paymentHelper;

    /**
     * @param ConfigFactory $configFactory
     * @param ResolverInterface $localeResolver
     * @param CurrentCustomer $currentCustomer
     * @param EtnpayHelper $paymentHelper
     * @param PaymentHelper $paymentHelper
     */
    public function __construct(
        \Psr\Log\LoggerInterface $logger,
        ConfigFactory $configFactory,
        ResolverInterface $localeResolver,
        CurrentCustomer $currentCustomer,
        EtnpayHelper $etnpayHelper,
        PaymentHelper $paymentHelper
    ) {
        $this->_logger          = $logger;
        $this->localeResolver   = $localeResolver;
        $this->config           = $configFactory->create();
        $this->currentCustomer  = $currentCustomer;
        $this->etnpayHelper     = $etnpayHelper;
        $this->paymentHelper    = $paymentHelper;

        foreach ( $this->methodCodes as $code ) {
            $this->methods[$code] = $this->paymentHelper->getMethodInstance( $code );
        }
    }

    /**
     * {@inheritdoc}
     */
    public function getConfig()
    {
        $config = [
            'payment' => [
                'etnpay' => [
                    'paymentAcceptanceMarkSrc'  => $this->config->getPaymentMarkImageUrl(),
                    'paymentAcceptanceMarkHref' => $this->config->getPaymentMarkWhatIsElectroneum(),
                    'paymentInstructions'       => $this->config->getPaymentInstructions(),
                    'etnAmountDetails'          => $this->etnpayHelper->getEtnValue(),
                ],
            ],
        ];

        foreach ( $this->methodCodes as $code ) {
            if ( $this->methods[$code]->isAvailable() ) {
                $config['payment']['etnpay']['redirectUrl'][$code]          = $this->getMethodRedirectUrl( $code );
                $config['payment']['etnpay']['billingAgreementCode'][$code] = $this->getBillingAgreementCode( $code );
            }
        }

        return $config;
    }

    /**
     * Return redirect URL for method
     *
     * @param string $code
     * @return mixed
     */
    protected function getMethodRedirectUrl( $code )
    {
        $methodUrl = $this->methods[$code]->getCheckoutRedirectUrl();
        return $methodUrl;
    }

    /**
     * Return billing agreement code for method
     *
     * @param string $code
     * @return null|string
     */
    protected function getBillingAgreementCode( $code )
    {
        $customerId = $this->currentCustomer->getCustomerId();
        $this->config->setMethod( $code );

        // Always return null
        return $this->etnpayHelper->shouldAskToCreateBillingAgreement( $this->config, $customerId );
    }
}
